@@ -29,14 +29,18 @@ import json
 import re
 import sys
 from pprint import pprint
+from warnings import warn
 from typing import Union, Mapping, Iterable, Generator
 from configparser import ConfigParser
 from functools import partial
 import os
+
 import argh
 import pytest
 import pylint.lint
 
+from wads.util import git
+
 CONFIG_FILE_NAME = 'setup.cfg'
 METADATA_SECTION = 'metadata'
 OPTIONS_SECTION = 'options'
@@ -51,31 +55,6 @@ DOCSRC = 'docsrc'
 DFLT_PUBLISH_DOCS_TO = None  # 'github'
 
 
-def git(command='status', work_tree='.', git_dir=None):
-    """Launch git commands.
-
-    :param command: git command (e.g. 'status', 'branch', 'commit -m "blah"', 'push', etc.)
-    :param work_tree: The work_tree directory (i.e. where the project is)
-    :param git_dir: The .git directory (usually, and by default, will be taken to be "{work_tree}/.git/"
-    :return: What ever the command line returns (decoded to string)
-    """
-
-    """
-
-    git --git-dir=/path/to/my/directory/.git/ --work-tree=/path/to/my/directory/ add myFile
-    git --git-dir=/path/to/my/directory/.git/ --work-tree=/path/to/my/directory/ commit -m 'something'
-
-    """
-    work_tree = os.path.abspath(os.path.expanduser(work_tree))
-    if git_dir is None:
-        git_dir = os.path.join(work_tree, '.git')
-    command = f'git --git-dir="{git_dir}" --work-tree="{work_tree}" {command}'
-    r = subprocess.check_output(command, shell=True)
-    if isinstance(r, bytes):
-        r = r.decode()
-    return r.strip()
-
-
 def get_name_from_configs(pkg_dir, assert_exists=True):
     """Get name from local setup.cfg (metadata section)"""
     configs = read_configs(pkg_dir=pkg_dir)
@@ -181,6 +160,7 @@ def check_in(
                 for subdir in next(os.walk(current_dir))[1]:
                     run_pylint(os.path.join(current_dir, subdir))
 
+        print_step_title('Validate docstrings')
         run_pylint('.')
 
     def run_tests():
@@ -214,7 +194,6 @@ def check_in(
         )
 
     def stage_changes():
-        print_step_title('Stage changes')
         result = ggit('status')
         if 'Changes not staged for commit' in result:
             if not auto_choose_default_action and not verbose:
@@ -222,6 +201,7 @@ def check_in(
             if confirm(
                 'Do you want to stage all your pending changes', default=True
             ):
+                print_step_title('Stage changes')
                 git('add -A')
 
     def commit_changes():
@@ -232,12 +212,12 @@ def check_in(
         ggit(f'commit --message="{commit_message}"')
 
     def push_changes():
-        print_step_title('Push changes')
         if not confirm(
             'Your changes have been commited. Do you want to push',
             default=True,
         ):
             abort()
+        print_step_title('Push changes')
         ggit('push')
 
     try:
@@ -315,6 +295,7 @@ def go(
     answer_yes_to_all_prompts: bool = False,
     twine_upload_options_str: str = '',
     keep_dist_pkgs=False,
+    commit_message='',
 ):
     """Update version, package and deploy:
     Runs in a sequence: increment_configs_version, update_setup_cfg, run_setup, twine_upload_dist
@@ -338,7 +319,11 @@ def go(
         generate_and_publish_docs(pkg_dir, publish_docs_to)
     if not skip_git_commit:
         git_commit_and_push(
-            pkg_dir, version, verbose, answer_yes_to_all_prompts
+            pkg_dir,
+            version,
+            verbose,
+            answer_yes_to_all_prompts,
+            commit_message,
         )
 
 
@@ -347,6 +332,7 @@ def git_commit_and_push(
     version=None,
     verbose: bool = True,
     answer_yes_to_all_prompts: bool = False,
+    commit_message='',
 ):
     def ggit(command):
         r = git(command, work_tree=pkg_dir)
@@ -363,19 +349,19 @@ def git_commit_and_push(
 
     ggit('status')  # show status again
 
+    commit_command = f'commit -m "{version}: {commit_message}"'
     if not answer_yes_to_all_prompts:
-        answer = input(f'Should I commit -m "{version}" and push? ([Y]/n)')
+        answer = input(f'Should I {commit_command} and push? ([Y]/n)')
         if answer and answer != 'Y':
             print("Okay, I'll stop here.")
             return
-    ggit(f'commit -m {version}')
+    ggit(commit_command)
     ggit(f'push')
 
 
 def generate_and_publish_docs(pkg_dir, publish_docs_to='github'):
     # TODO: Figure out epythet and wads relationship -- right now, there's a reflexive dependency
-    from epythet.docs_gen.autogen import make_autodocs
-    from epythet.docs_gen.call_make import make
+    from epythet import make_autodocs, make
 
     make_autodocs(pkg_dir)
     if publish_docs_to:
