@@ -1,11 +1,25 @@
 import re
 from itertools import chain
 import pkgutil
+from warnings import warn
 from functools import partial
 import os
 import subprocess
 
 standard_lib_dir = os.path.dirname(os.__file__)
+path_sep = os.path.sep
+
+
+def ensure_slash_suffix(s: str):
+    if not s.endswith(path_sep):
+        s += path_sep
+    return s
+
+
+def ensure_no_slash_suffix(s: str):
+    if s.endswith(path_sep):
+        s = s[:-1]
+    return s
 
 
 def git(command='status', work_tree='.', git_dir=None):
@@ -18,16 +32,28 @@ def git(command='status', work_tree='.', git_dir=None):
     """
 
     """
-    
+
     git --git-dir=/path/to/my/directory/.git/ --work-tree=/path/to/my/directory/ add myFile
     git --git-dir=/path/to/my/directory/.git/ --work-tree=/path/to/my/directory/ commit -m 'something'
 
     """
+    if command.startswith('git '):
+        warn(
+            "You don't need to start your command with 'git '. I know it's a git command. Removing that prefix"
+        )
+        command = command[len('git ') :]
     work_tree = os.path.abspath(os.path.expanduser(work_tree))
     if git_dir is None:
         git_dir = os.path.join(work_tree, '.git')
+    assert os.path.isdir(git_dir), f"Didn't find the git_dir: {git_dir}"
+    git_dir = ensure_no_slash_suffix(git_dir)
+    if not git_dir.endswith('.git'):
+        warn(f"git_dir doesn't end with `.git`: {git_dir}")
     command = f'git --git-dir="{git_dir}" --work-tree="{work_tree}" {command}'
-    return subprocess.check_output(command, shell=True).decode()
+    r = subprocess.check_output(command, shell=True)
+    if isinstance(r, bytes):
+        r = r.decode()
+    return r.strip()
 
 
 def is_standard_lib_path(path):
